// Copyright (C) 2008 - 2010 - Michael Baudin

function c = flps_chop ( varargin )
  //   Round matrix elements to t significant binary places.
  //
  // Calling Sequence
  // c = chop ( x )
  // c = chop ( x , t )
  // c = chop ( x , t , b )
  //
  // Parameters
  //   x : the matrix to chop
  //   t : the number of significant bits to chop. Default is t = 24, corresponding to IEEE single precision.
  //   b : the radix
  //
  // Description
  //   This function allows to simulate a system with a 
  //   lower precision than doubles. 
  //
  // Examples
  //  c = flps_chop ( 1/3 )
  //  c = flps_chop ( [1/3 0 0 1 2 1/5] )
  //  c = flps_chop ( [1/3 0 0 1 2 1/5] , 48 )
  //  c = flps_chop ( 0.123456789 , 3 , 10 )
  //
  // Authors
  // Nicolas Higham, Matlab Version
  // Michael Baudin, 2010, Scilab port
  //
  // Bibliography
  //   "The Test Matrix Toolbox for Matlab (Version 3.0)", Nicolas Higham, Numerical Analysis Report No. 276, September 1995, Manchester Centre for Computational Mathematics, The University of Manchester
  //   "Higham's Test Matrices ", John Burkardt, 2005, http://people.sc.fsu.edu/~burkardt/m_src/test_matrix/test_matrix.html
  //

  [lhs,rhs]=argn();
  if ( rhs < 1 | rhs > 3 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while from 1 to 3 are expected."), "flps_chop", rhs);
    error(errmsg)
  end
  x = varargin(1)
  if ( rhs < 2 ) then
    t = 24
  else
    t = varargin(2)
  end
  if ( rhs < 3 ) then
    b = 2
  else
    b = varargin(3)
  end
  //  Use the representation:
  //  x(i,j) = 2^e(i,j) * .d(1)d(2)...d(s) * sign(x(i,j))
  nz = find(x<>0)
  y(nz) = abs(x(nz));
  e(nz) = floor(log(y(nz))/log(b) + 1);
  p(nz) = x(nz).*b^(t-e(nz))
  c(nz) = round(p(nz)) .* b .^ (e(nz)-t);
  zz = find(x==0)
  c(zz) = 0
endfunction

