// Copyright (C) 2008 - 2010 - Michael Baudin

function [ emax , vmax ] = flps_emax ( radix )
  //   Returns the maximum exponent and value before overflow.
  //
  // Calling Sequence
  //   [ emax , vmax ] = flps_emax ( radix )
  //
  // Parameters
  //   radix: the basis
  //   emax : maximum exponent before overflow
  //   vmax : maximum value before overflow
  //
  // Description
  //   This algorithm is based on a loop, with an 
  //   increasing value t = radix * s. If t/radix is different
  //   from s, then the overflow threshold has been 
  //   detected.
  //
  // Examples
  // [radix,rounding] = flps_radix ();
  // [ emax , vmax ] = flps_emax ( radix )
  // number_properties( "huge" )
  // number_properties( "maxexp" )
  //
  // Authors
  // Michael Baudin, 2008 - 2010
  //
  // Bibliography
  //   "Algorithms to Reveal Properties of Floating-Point Arithmetic", Michael A. Malcolm, Stanford University, Communications of the ACM, Volume 15,  Issue 11 (November 1972), Pages: 949 - 951
  //   "More on Algorithms that Reveal Properties of Floating Point Arithmetic Units", W. Morven Gentleman, University of Waterloo, Scott B. Marovich, Purdue University, Communications of the ACM, Volume 17,  Issue 5 (May 1974), Pages: 276 - 277
  //   "ALGORITHM 665 - MACHAR: A Subroutine to Dynamically Determine Machine Parameters", W. J. Cody, Argonne National Laboratory, ACM Transactions on Mathematical Software, Vol. 14, No. 4, December 1988, Pages 303-311.
  //
  
  vmax = 1.
  emax = 0
  while ( %t )
    emax = emax + 1
    // Condition #1 : no exception is generated
    n = vmax * radix
    // Condition #2 : one can recover the original number
    if ( n / radix <> vmax) then
      break
    end
    vmax = n
  end
  //
  // Our definition is 
  //    x = M * beta ^ ( e - p + 1 )
  // where M is the integral significant in the interval [0,beta^p - 1[
  // and e is the exponent in the interval emin <= e <= emax.
  // Because of this definition, we must decrease emin by 1.
  // This does not change vmin.
  //
  emax = emax - 1
endfunction

