// Copyright (C) 2008 - 2010 - Michael Baudin

function [ f , e ] = flps_frexp ( varargin )
  //   Returns the exponent and fraction.
  //
  // Calling Sequence
  // [ f , e ] = flps_frexp ( b , x )
  // [ f , e ] = flps_frexp ( b , x , algo )
  //
  // Parameters
  //   b : the radix
  //   x : the floating point number to decompose
  //   algo : "v1" or "v2" (default is "v1"). If algo="v1", then, on output, f satisfies 1/b <= f < 1. If algo="v2", then, on output, f satisfies 1 <= f < b.
  //   f : the fraction
  //   e : the exponent
  //
  // Description
  //   The exponent and the fraction are so that x = f * b^e.
  //   See also Scilab's function frexp.
  //
  // Examples
  //   [ f , e ] = flps_frexp ( 2 , 1 )
  //   [ f , e ] = flps_frexp ( 3 , 1 )
  //   [ f , e ] = flps_frexp ( 3 , 1 , "v1" )
  //   [ f , e ] = flps_frexp ( 3 , 1 , "v2" )
  //
  // Authors
  // Michael Baudin, 2008 - 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010

  [lhs,rhs]=argn();
  if ( rhs < 2 | rhs > 3 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while at least 2 to 3 are expected."), "flps_frexp", rhs);
    error(errmsg)
  end
  b = varargin(1)
  x = varargin(2)
  if ( rhs == 3 ) then
    algo = varargin(3)
  else
    algo = "v1"
  end
  
  select algo
  case "v1" then
    // Compute the log-b of |x|
    l = log(abs(x))./log(b)
    e = floor(l) + 1
    f = x./b.^e
  case "v2" then
    l = log(abs(x))./log(b)
    e = floor(l)
    f = x./b.^e
  else
    errmsg = msprintf(gettext("%s: Unrecognized matrix name ''%s''."), "flps_frexp", algo )
    error(errmsg)
  end
endfunction

