// Copyright (C) 2008 - 2010 - Michael Baudin

function x = flps_frombary ( b , d )
  // Returns the floating point number given its b-ary decomposition.
  //
  // Calling Sequence
  // x = flps_frombary ( b , d )
  //
  // Parameters
  //   b : the radix
  //   d : the digits in lower endian order.
  //   x : the floating point number to decompose
  //
  // Description
  //   Given the radix b and the digits d(i), for i = 1, 2, ..., p,
  //   returns a number
  //   x = d(1) + d(2)/b + d(3)/b^2 + ... + d(p-1)/b^(b-1) such that
  //   0 <= x < b.
  //
  //
  // Examples
  // // This is x = 1
  // x = flps_frombary ( 2 , [1 0]' )
  // // This is x = 1/3+1/3^3
  // x = flps_frombary ( 3 , [0 1 0 1]' )
  // // This is x = 1 + 1/2 + 1/2^3 + 1/2^5
  // x = flps_frombary ( 2 , [1 1 0 1 0 1]' )
  //
  // Authors
  // Michael Baudin, 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010

  if ( b < 2 ) then
    errmsg = msprintf(gettext("%s: Unexpected radix b = %d. Expect 2<= b."), "flps_frombary", b);
    error(errmsg)
  end
  x = 0
  q = 1
  p = size(d,"*")
  for i = 1 : p
    x = x + d(i) * q
    q = q/b
  end
endfunction

