// Copyright (C) 2008 - 2010 - Michael Baudin

function f = flps_numbereval ( flpn )
  //   Returns the value of the current floating point number.
  //
  // Calling Sequence
  //   f = flps_numbereval ( flpn )
  //
  // Parameters
  //   flpn: a floating point number, or a list of floating point numbers
  //   f: the value
  //
  // Description
  //   Returns an evaluation of the floating point number flpn.
  //   If flpn is a list,  returns a column matrix containing the 
  //   values of the floating point numbers in this list.
  //
  // Examples
  //   // Evaluate one floating point number
  //   x = 4;
  //   r = 2;
  //   p = 3;
  //   e = 3;
  //   flps = flps_format2system ( r , p , e )
  //   flpn = flps_numberformat ( flps , x )
  //   // This should be f = 4
  //   f = flps_numbereval ( flpn )
  //
  //   // Evaluate all the floating point numbers in a system
  //   flps = flps_format2system ( 2 , 3 , 3 );
  //   listflpn = flps_systemall ( flps );
  //   This should be -14, -12, -10, -8, ..., 12, 14
  //   f = flps_numbereval ( listflpn )
  //
  // Authors
  // Michael Baudin, 2008 - 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010

  if ( typeof ( flpn ) == "TFLNUMB" ) then
    flps = flpn.flps
    f = flpn.M * flps.radix^(flpn.e - flps.p + 1)
  elseif ( typeof ( flpn ) == "list" ) then
    n = size(flpn);
    f = zeros(n,1)
    for i = 1 : n
      if ( typeof ( flpn(i) ) <> "TFLNUMB" ) then
        errmsg = msprintf(gettext("%s: Element #%d in the list is not a floating point number."),"flps_numbereval",i)
        error ( errmsg )
      end
      f(i) = flps_numbereval ( flpn(i) )
    end
  else
    errmsg = msprintf(gettext("%s: Unknown type %s for input argument flpn.") , "flps_numbereval" , typeof ( flpn ) )
    error ( errmsg )
  end
endfunction

