// Copyright (C) 2008 - 2010 - Michael Baudin

function flpn = flps_numberformat ( flps , x )
  //   Returns the floating point number of the given double.
  //
  // Calling Sequence
  //   flpn = flps_numberformat ( flps , x )
  //
  // Parameters
  //   flps : a floating point system
  //   x : the floating point number (a double)
  //   flpn: a floating point number
  //
  // Description
  //   Returns the floating point number of the given double x
  //   on the given floating point system flps.
  //   The returned floating point number has been created with 
  //   flnumber_new().
  //   This function takes into account for the rounding modes associated 
  //   with flps.
  //   The round to nearest mode is associated with the round ties 
  //   to even rule.
  //   There is no management for exceptions as underflows and overflows.
  //   TODO : manage exceptions
  //
  // Examples
  // flps = flps_IEEEsingle ( );
  // flpn = flps_numberformat ( flps , 1/3 )
  // flpn = flps_numberformat ( flps , 3*2^-128 )
  // // Check rounding modes : default mode is round to nearest
  // flps = flps_format2system ( 2 , 3 , 3 );
  // flpn = flps_numberformat ( flps , 0.54 )
  // flpn = flps_numberformat ( flps , 0.60 )
  // flpn = flps_numberformat ( flps , -0.54 )
  // flpn = flps_numberformat ( flps , -0.60 )
  // // A case of tie: check that round to even is applied
  // x= 4.5 * 2^(-1-3+1);
  // flpn = flps_numberformat ( flps , x )
  // // A case of tie: check that round to even is applied
  // x= 4.5 * 2^(0-3+1);
  // flpn = flps_numberformat ( flps , x )
  // // A case of tie: check that round to even is applied
  // x= -4.5 * 2^(0-3+1);
  // flpn = flps_numberformat ( flps , x )
  // // A case where the number is rounded up, which makes the exponent increase
  // // to keep the number normalized
  // flpn = flps_numberformat ( flps , 0.49 )
  // // Check rounding modes : set rounding mode to round up
  // flps = flps_format2system ( 2 , 3 , 3 );
  // flps.r = 2;
  // flpn = flps_numberformat ( flps , 0.54 )
  // flpn = flps_numberformat ( flps , 0.60 )
  // flpn = flps_numberformat ( flps , -0.54 )
  // flpn = flps_numberformat ( flps , -0.60 )
  // // A case where the number is rounded up, which makes the exponent increase
  // // to keep the number normalized
  // flpn = flps_numberformat ( flps , 0.99 )
  // // Check rounding modes : set rounding mode to round down
  // flps = flps_format2system ( 2 , 3 , 3 );
  // flps.r = 3;
  // flpn = flps_numberformat ( flps , 0.54 )
  // flpn = flps_numberformat ( flps , 0.60 )
  // flpn = flps_numberformat ( flps , -0.54 )
  // flpn = flps_numberformat ( flps , -0.60 )
  // // A case where the number is rounded up, which makes the exponent decrease
  // // to keep the number normalized
  // flpn = flps_numberformat ( flps , -0.49 )
  // // Check rounding modes : set rounding mode to round to zero
  // flps = flps_format2system ( 2 , 3 , 3 );
  // flps.r = 4;
  // flpn = flps_numberformat ( flps , 0.54 )
  // flpn = flps_numberformat ( flps , 0.60 )
  // flpn = flps_numberformat ( flps , -0.54 )
  // flpn = flps_numberformat ( flps , -0.60 )
  //
  // Authors
  // Michael Baudin, 2008 - 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010
  
  flpn = flps_numbernew ( )
  [ f , e ] = flps_frexp ( flps.radix , x , "v2" )
  flpn.e = min(max(e,flps.emin),flps.emax)
  if ( x > 0 ) then
    flpn.s = 0
  else
    flpn.s = 1
  end
  select flps.r
  case 1
    // Rounding= To Nearest
    // Compute the infinitely precise significant
    Minf = x / flps.radix^(flpn.e-flps.p+1)
    // Compute the two integral significant which are closest to Minf: M1 < Minf < M2
    M1 = floor(Minf)
    M2 = ceil(Minf)
    if ( M2-Minf <> Minf - M1 ) then
      // If there is no tie, round to nearest
      M = round(Minf)
      if ( abs(M) < flps.radix^flps.p ) then
        flpn.M = M
      else
        // If rounding to nearest is up and the number becomes denormal
        // increase the exponent.
        flpn.M = M/2
        flpn.e = flpn.e + 1
      end
    else
      // Select the even one
      if ( modulo(M1,2)==0 ) then
        flpn.M = M1
      else
        flpn.M = M2
      end
    end
  case 2
    // Rounding= Up
    M = ceil(x / flps.radix^(flpn.e-flps.p+1))
    // If rounding to nearest is up and the number becomes denormal
    // increase the exponent.
    if ( abs(M) < flps.radix^flps.p ) then
      flpn.M = M
    else
      flpn.M = M/2
      flpn.e = flpn.e + 1
    end
  case 3
    // Rounding= Down
    M = floor(x / flps.radix^(flpn.e-flps.p+1))
    if ( abs(M) < flps.radix^flps.p ) then
      flpn.M = M
    else
      flpn.M = M/2
      flpn.e = flpn.e + 1
    end
  case 4
    // Rounding= To Zero
    flpn.M = int(x / flps.radix^(flpn.e-flps.p+1))
  else
    errmsg = msprintf(gettext("%s: Unexpected rounding mode: %d."), "flps_numberformat", flps.r);
    error(errmsg)
  end
  flpn.m = abs(flpn.M) * flps.radix^(1-flps.p)
  flpn.d = flps_tobary ( flpn.m , flps.radix , flps.p )
  flpn.flps = flps
endfunction

