// Copyright (C) 2008 - 2010 - Michael Baudin

function flpn = flps_numbernew ( varargin )
  //   Returns a new floating point number.
  //
  // Calling Sequence
  //   flpn = flps_numbernew ( )
  //   flpn = flps_numbernew ( "integral" , flps , M , e )
  //   flpn = flps_numbernew ( "signm" , flps , s , m , e)
  //
  // Parameters
  //   rtype: the type of representation, "integral" or "signm"
  //   flps : a floating point system
  //   x : the floating point number (a double)
  //   flpn: a floating point number
  //   flpn.sign: 0 if x is positive, 1 if x is negative
  //   flpn.M : a signed integer, the integral significant, so that x = M * b^(e-p+1)
  //   flpn.m : an unsigned real, the normal significant, so that x = (-1)^s * m * b^e and 0 <= m < beta
  //   flpn.e : a signed integer, the exponent
  //   flpn.d : the digits from the b-ary decomposition of m (little endian order)
  //   flpn.flps : the floating point system supporting the number
  //
  // Description
  //   If the "integral" representation is chosen, 
  //   returns a new floating point number with given (M,e) representation.
  //   The sign s is computed by the rule s=0 is for x >= 0,
  //   s=1 is for x < 0.
  //   The normal significant is computed by the rule m = |M| * radix^(1-p).
  //   No negative zero can be produced with this "integral" constructor.
  //
  //   Returns a new floating point number with given (s,m,e) representation.
  //   The integral significant is computed by the rule M = (-1)^s * m * radix^(p - 1).
  //
  //   TODO : manage exceptions
  //
  // Examples
  // // A toy system, including denormals and negative
  // r = 2;
  // p = 3;
  // e = 3;
  // flps = flps_format2system ( r , p , e );
  // flpn = flps_numbernew ( "integral" , flps , 4 , 2 )
  // string(flpn)
  // // A toy system, including denormals and negative
  // r = 2;
  // p = 3;
  // e = 3;
  // flps = flps_format2system ( r , p , e );
  // flpn = flps_numbernew ( "signm" , flps , 0 , 1 , 2 )
  // string(flpn)
  //
  // Authors
  // Michael Baudin, 2008 - 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010


  [lhs,rhs]=argn();
  if ( rhs > 5 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while from 1 to 4 are expected."), "flps_numbernew", rhs);
    error(errmsg)
  end
  flpn = tlist(["TFLNUMB"
    "s"
    "M"
    "m"
    "e"
    "d"
    "flps"
    ]);
  if ( rhs == 0 ) then
    flpn.s = [];
    flpn.M = [];
    flpn.m = [];
    flpn.e = [];
    flpn.d = [];
    flpn.flps = flps_systemnew ( );
    return
  end
  rtype = varargin(1)
  if ( rtype == "integral" ) then
    if ( rhs <> 4 ) then
      errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while 4 are expected."), "flps_numbernew", rhs);
      error(errmsg)
    end
    flps = varargin(2)
    M = varargin(3)
    e = varargin(4)
    flpn.flps = flps
    flpn.e = e
    if ( M >= 0 ) then
      flpn.s = 0
    else
      flpn.s = 1
    end
    flpn.M = M
    flpn.m = abs(M) * flps.radix^(1-flps.p)
    flpn.d = flps_tobary ( flpn.m , flps.radix , flps.p )
  elseif ( rtype == "signm" ) then
    if ( rhs <> 5 ) then
      errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while 5 are expected."), "flps_numbernew", rhs);
      error(errmsg)
    end
    flps = varargin(2)
    flpn.flps = flps
    s = varargin(3)
    m = varargin(4)
    e = varargin(5)
    if ( s <> 0 & s <> 1 ) then
      errmsg = msprintf(gettext("%s: Unexpected value of s : %d provided while 0 or 1 is expected."), "flps_numbernew", s);
      error(errmsg)
    end
    flpn.s = s
    flpn.m = m
    flpn.e = e
    flpn.M = (-1)^s * m * flps.radix^(flps.p - 1)
    flpn.d = flps_tobary ( flpn.m , flps.radix , flps.p )
  else
    errmsg = msprintf(gettext("%s: Unexpected representation type: %s provided while ''integral'' or ''signm'' are expected."), "flps_numbernew", string(rtype));
    error(errmsg)
  end
endfunction

