// Copyright (C) 2010 - Michael Baudin

function listflpn = flps_systemall ( varargin )
  //   Returns the list of floating point numbers of the given floating point system.
  //
  // Calling Sequence
  //  listflpn = flps_systemall ( flps )
  //  listflpn = flps_systemall ( flps , denormals )
  //  listflpn = flps_systemall ( flps , denormals , onlypos )
  //
  // Parameters
  //   flps : a floating point system
  //   denormals : a boolean. Set to true to include denormalized numbers. Set to false to ignore denormalized numbers. Default is %t.
  //   onlypos : a boolean. Set to true to include only positive numbers. Set to false to include both positive and negative numbers. Default is %f.
  //   listflpn: the list of normalized floating point number
  //
  // Description
  //   This function allows to get a list of floating point numbers
  //   of a toy floating point system.
  //   
  //
  // Examples
  //   r = 2;
  //   p = 3;
  //   e = 3;
  //   flps = flps_format2system ( r , p , e );
  //   // There are 56 f.p. numbers
  //   listflpn = flps_systemall ( flps );
  //   x = flps_numbereval ( listflpn )'
  //   // Ignore denormals
  //   listflpn = flps_systemall ( flps , %f );
  //   x = flps_numbereval ( listflpn )'
  //   // Get only positive numbers, ignore denormals
  //   listflpn = flps_systemall ( flps , %f , %t );
  //   x = flps_numbereval ( listflpn )'
  //
  // Authors
  // Michael Baudin, 2008 - 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010

  [lhs,rhs]=argn();
  if ( rhs < 1 | rhs > 3 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while from 1 to 3 are expected."), "flps_systemall", rhs);
    error(errmsg)
  end
  flps = varargin(1)
  if ( rhs < 2 ) then
    denormals = %t
  else
    denormals = varargin(2)
  end
  if ( rhs < 3 ) then
    onlypos = %f
  else
    onlypos = varargin(3)
  end
  Mmin = flps.radix^(flps.p - 1)
  Mmax = flps.radix^flps.p - 1
  listflpn = list()
  if ( ~onlypos ) then
    // Negative normalized numbers
    for e = flps.emax : -1 : flps.emin
      for M = -Mmax : -Mmin
        listflpn($+1) = flps_numbernew ( "integral" , flps , M , e )
      end
    end
    // Negative denormalized numbers
    if ( denormals ) then
      e = flps.emin
      for M = -Mmin + 1 : -1
        listflpn($+1) = flps_numbernew ( "integral" , flps , M , e )
      end
    end
    // Add -zero
    flpn = flps_numbernew ( "integral" , flps , 0 , 0 )
    // Force negative sign
    flpn.s = 1
    listflpn($+1) = flpn
  end
  // Add +zero
  listflpn($+1) = flps_numbernew ( "integral" , flps , 0 , 0 )
  // Positive denormalized numbers
  if ( denormals ) then
    e = flps.emin
    for M = 1 : Mmin - 1
      listflpn($+1) = flps_numbernew ( "integral" , flps , M , e )
    end
  end
  // Positive normalized numbers
  for e = flps.emin : flps.emax
    for M = Mmin : Mmax
      listflpn($+1) = flps_numbernew ( "integral" , flps , M , e )
    end
  end
endfunction


