// Copyright (C) 2008 - 2010 - Michael Baudin

function h = flps_systemgui ( varargin )
  //   Plots all the numbers in the current floating point system
  // Calling Sequence
  //  h = flps_systemgui ( flps )
  //  h = flps_systemgui ( flps , denormals )
  //  h = flps_systemgui ( flps , denormals , onlypos )
  //  h = flps_systemgui ( flps , denormals , onlypos , logscale )
  //
  // Parameters
  //   flps : a floating point system
  //   denormals : a boolean. Set to true to include denormalized numbers. Set to false to ignore denormalized numbers. Default is %t.
  //   onlypos : a boolean. Set to %t to include only positive numbers. Set to %f to include both positive and negative numbers. Default is %f.
  //   logscale : a boolean. Set to %t to use a base-10 logarithmic scale. Set to %f to see the actual values. Default is %f.
  //
  // Description
  //   This function is used to see the distribution of floating point numbers.
  //
  // Examples
  // // A toy system, including denormals and negative
  // r = 2;
  // p = 3;
  // e = 3;
  // flps = flps_format2system ( r , p , e );
  // h = flps_systemgui ( flps );
  // // Wait : see the picture before closing it...
  // close(h);
  //
  // Authors
  // Michael Baudin, 2010
  //
  // Bibliography
  //   "Numerical Computing with MATLAB", Cleve Moler, chapter "Introduction to MATLAB", section 1.7 "Floating point arithmetic"
  //   "Accuracy and Stability of Numerical Algorithms", Nicolas Higham, SIAM, 2002
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010

  [lhs,rhs]=argn();
  if ( rhs < 1 | rhs > 4 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while from 1 to 4 are expected."), "flps_systemgui", rhs);
    error(errmsg)
  end
  flps = varargin(1)
  if ( rhs < 2 ) then
    denormals = %t
  else
    denormals = varargin(2)
  end
  if ( rhs < 3 ) then
    onlypos = %f
  else
    onlypos = varargin(3)
  end
  if ( rhs < 4 ) then
    logscale = %f
  else
    logscale = varargin(4)
  end
  flpn = flps_systemall ( flps , denormals , onlypos );
  f = flps_numbereval ( flpn );
  if ( logscale == %t ) then
    // Remove zeros
    ii = find ( f == 0 )
    f(ii) = []
  end
  n = size(f,"r");
  drawlater();
  for i = 1 : n
    plot([f(i) f(i)],[-1 1],"b-")
  end
  xmin = 1.05 * min(f)
  xmax = 1.05 * max(f)
  h = gcf()
  h.children.data_bounds = [
    xmin -3
    xmax 3
  ];
  h.children.axes_visible= ["on" "off" "on"]
  if ( logscale == %t ) then
    h.children.log_flags = "lnn"
  end
  h.figure_size = [600 300]
  drawnow();
endfunction

