// Copyright (C) 2008 - 2010 - Michael Baudin

function flps = flps_systemnew ( varargin )
  //   Returns a new floating point system.
  //
  // Calling Sequence
  // flps = flps_systemnew ( )
  // flps = flps_systemnew ( fill )
  //
  // Parameters
  //   fill : if fill="empty", then empty fields are returned. If fill="current", then compute the current properties of the system and fill the fields with them. (Default is fill="empty")
  //   flps : an empty floating point system
  //   flps.radix: the basis for floating point numbers
  //   flps.p : the precision
  //   flps.emin : the minimum exponent before underflow
  //   flps.vmin : the minimum value before underflow
  //   flps.emax : the maximum exponent before overflow
  //   flps.vmax : the maximum value before overflow
  //   flps.eps : the smallest value so that 1+eps <> 1 (the machine epsilon)
  //   flps.r : the rounding mode. The value r = 1 means round-to-nearest, r=2 means round-up, r=3 means round-down, r=4 means round-to-zero.
  //   flps.gu : %t if the machine supports graceful underflow. False if not. IEEE machines must support graceful underflow.
  //   flps.alpha : the smallest positive subnormal number
  //
  // Description
  //   Note:we do not include the unit roundoff u.
  //   If we did, and if the user changes the rounding mode,
  //   then the unit roundoff should be updated accordingly.
  //   Since there is currently no system to update u automatically,
  //   it is safer not to store u.
  //
  //   If type="current", compute automatically the radix and rounding mode for Scilab doubles.
  //   Caution: the current detected rounding mode is only round-to-nearest.
  //   Other rounding modes are not detected by this function, i.e. 
  //   if flps.r is not equal to 1, it is equal to %nan.
  //   Compute automatically the machine epsilon and the precision for Scilab doubles.
  //   Compute automatically the maximum value and exponent.
  //   Compute automatically the minimum value and exponent and the 
  //   graceful underflow field. 
  //
  // Examples
  // flps = flps_systemnew ()
  // flps.radix = 2
  // flps.p = 12
  // string(flps)
  //
  // // Compute the current floating point properties
  // flps = flps_systemnew ( "current" )
  //
  // Authors
  // Michael Baudin, 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010
  
  [lhs,rhs]=argn();
  if ( rhs > 1 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while from 0 or 1 are expected."), "flps_systemnew", rhs);
    error(errmsg)
  end
  if ( rhs == 0 ) then
    fill = "empty"
  else
    fill = varargin(1)
  end
  if ( and ( fill <> ["empty" "current"] ) ) then
    errmsg = msprintf(gettext("%s: Unexpected fill value : %s provided while ""empty"" or ""current"" are expected."), "flps_systemnew", fill);
    error(errmsg)
  end
  
  flps = tlist(["TFLSYS"
    "radix"
    "p"
    "emin"
    "emax"
    "vmin"
    "vmax"
    "eps"
    "r"
    "gu"
    "alpha"
    ]);
  if ( fill == "empty" ) then
    flps.radix = []
    flps.p = []
    flps.emin = []
    flps.emax = []
    flps.vmin = []
    flps.vmax = []
    flps.eps = []
    flps.r = []
    flps.gu = []
    flps.alpha = []
  elseif ( fill == "current" ) then
    [ radix , rounding ] = flps_radix ();
    [eps,p] = flps_eps (radix);
    [ emax , vmax ] = flps_emax ( radix );
    [ emin , vmin , gu , alpha ] = flps_emin ( radix , p );
    flps.radix = radix
    flps.p = p
    flps.emin = emin
    flps.emax = emax
    flps.eps = eps
    flps.vmin = vmin
    flps.vmax = vmax
    if ( rounding ) then
      flps.r = 1
    else
      flps.r = %nan
    end
    flps.gu = gu
    flps.alpha = alpha
  end
endfunction

