// Copyright (C) 2008 - 2010 - Michael Baudin

function d = flps_tobary ( x , b , p )
  //  Returns the digits of the b-ary decomposition. 
  //
  // Calling Sequence
  // d = flps_tobary ( x , b , p )
  //
  // Parameters
  //   x : the floating point number to decompose
  //   b : the radix
  //   p : the precision
  //   d : a p-by-1 matrix, the digits of the b-ary decomposition of x. 
  //
  // Description
  //   Returns the digits of the b-ary decomposition of x in base b with 
  //   precision p.
  //   Given a number x and a radix b such that 
  //   0 <= x < b, and given a precision p,
  //   returns the digits d(i), for i = 1, 2, ..., p,
  //   with x = d(1) + d(2)/b + d(3)/b^2 + ... + d(p-1)/b^(b-1).
  //   Caution : there is no specific rounding mode guaranteed
  //   by this function, which implies that the last digit
  //   is most of the times unsafe.
  //
  // Examples
  //   // This is d = [1 0]
  //   d = flps_tobary ( 1 , 2 , 2 )
  //   // This is d = [0 1 0 1]
  //   x = 1/3+1/3^3;
  //   d = flps_tobary ( x , 3 , 4 );
  //   // This is d = [1 1 0 1 0 1]
  //   x = 1 + 1/2 + 1/2^3 + 1/2^5;
  //   d = flps_tobary ( x , 2 , 6 );
  //
  // Authors
  // Michael Baudin, 2010
  //
  // Bibliography
  //   "Handbook of floating-point arithmetic", Jean-Michel Muller, Nicolas Brisebarre, Florent de Dinechin, Claude-Pierre Jeannerod, Vincent Lefevre, Guillaume Melquiond, Nathalie Revol, Damien Stehle, Serge Torres, Birkhauser Boston, 2010
  
  if ( b < 2 ) then
    errmsg = msprintf(gettext("%s: Unexpected radix b = %d. Expect 2<= b."), "flps_tobary", b);
    error(errmsg)
  end
  if ( p < 1 ) then
    errmsg = msprintf(gettext("%s: Unexpected precision p = %d. Expect 1<= p."), "flps_tobary", p);
    error(errmsg)
  end
  if ( x < 0 | x>= b ) then
    errmsg = msprintf(gettext("%s: Unexpected x = %e. Expect 0<= x < %d."), "flps_tobary", x,b);
    error(errmsg)
  end
  q = 1
  for i = 1 : p
    d(i) = floor(x/q)
    x = x - d(i) * q
    q = q/b
  end
endfunction

